@@ -26,16 +26,32 @@ extern "C" SEXP luajr_run_parallel(SEXP func, SEXP n, SEXP threads, SEXP pre)
     CheckSEXPLen(n, INTSXP, 1);
     CheckSEXPLen(pre, STRSXP, 1);
 
+    // For any call to luajr_pcall
+    static const int tflags = LUAJR_NO_PROFILE_COLLECT | LUAJR_NO_ERROR_HANDLING | LUAJR_TOOLING_ALL;
+
     // Ensure n is sensible
     int n_iter = INTEGER(n)[0];
     if (n_iter < 0) // also covers NA_INTEGER
         Rf_error("Invalid number of iterations.");
 
+    // Don't multi-thread in debug mode
+    bool single_thread = false;
+    if (luajr_debug_mode())
+    {
+        single_thread = true;
+        Rf_warningcall_immediate(R_NilValue, "luajr debugger is active, so lua_parallel will only use one thread.");
+    }
+    else if (luajr_profile_mode())
+    {
+        single_thread = true;
+        Rf_warningcall_immediate(R_NilValue, "luajr profiler is active, so lua_parallel will only use one thread.");
+    }
+
     // Create or get Lua states for each thread
     std::vector<lua_State*> l;
     if (TYPEOF(threads) == INTSXP && Rf_length(threads) == 1)
     {
-        int n_threads = INTEGER(threads)[0];
+        int n_threads = single_thread ? 1 : INTEGER(threads)[0];
         if (n_threads <= 0) // also covers NA_INTEGER
             Rf_error("Invalid number of threads.");
         l.assign(n_threads, 0);
@@ -44,7 +60,7 @@ extern "C" SEXP luajr_run_parallel(SEXP func, SEXP n, SEXP threads, SEXP pre)
     }
     else if (TYPEOF(threads) == VECSXP && Rf_length(threads) > 0)
     {
-        l.assign(Rf_length(threads), 0);
+        l.assign(single_thread ? 1 : Rf_length(threads), 0);
         for (unsigned int t = 0; t < l.size(); ++t)
         {
             l[t] = luajr_getstate(VECTOR_ELT(threads, t));
@@ -78,12 +94,15 @@ extern "C" SEXP luajr_run_parallel(SEXP func, SEXP n, SEXP threads, SEXP pre)
         // Run pre-code
         if (pre_code != 0)
         {
-            if (luaL_dostring(l[t], pre_code))
+            int pre_code_error = luaL_loadstring(l[t], pre_code);
+            if (!pre_code_error)
+                pre_code_error = luajr_pcall(l[t], 0, 0, 0, tflags); // Discard any return values
+            if (pre_code_error)
             {
                 std::lock_guard<std::mutex> lock { pm };
-                error_msg = lua_tostring(l[t], -1);
+                error_msg.assign(1024, ' ');
+                luajr_handle_lua_error(l[t], pre_code_error, "lua_parallel 'pre' execution", error_msg.data());
             }
-            lua_settop(l[t], 0); // Discard any return values
         }
 
         // Has any thread produced an error?
@@ -92,7 +111,9 @@ extern "C" SEXP luajr_run_parallel(SEXP func, SEXP n, SEXP threads, SEXP pre)
 
         // Run command to get function on stack
         int top0 = lua_gettop(l[t]);
-        int err = luaL_dostring(l[t], cmd.c_str());
+        int err = luaL_loadstring(l[t], cmd.c_str());
+        if (!err)
+            err = luajr_pcall(l[t], 0, LUA_MULTRET, 0, tflags);
         int nret = lua_gettop(l[t]) - top0;
 
         // Handle errors
@@ -100,7 +121,8 @@ extern "C" SEXP luajr_run_parallel(SEXP func, SEXP n, SEXP threads, SEXP pre)
             // This and similar mutex locks are used to avoid writing
             // to error_message in multiple threads simultaneously.
             std::lock_guard<std::mutex> lock { pm };
-            error_msg = lua_tostring(l[t], -1);
+            error_msg.assign(1024, ' ');
+            luajr_handle_lua_error(l[t], err, "lua_parallel 'func' construction", error_msg.data());
         } else if (nret != 1) {
             std::lock_guard<std::mutex> lock { pm };
             error_msg = "lua_parallel expects `func' to evaluate to one value, not " +
@@ -125,13 +147,14 @@ extern "C" SEXP luajr_run_parallel(SEXP func, SEXP n, SEXP threads, SEXP pre)
             int top1 = lua_gettop(l[t]);
             lua_pushvalue(l[t], top0);
             lua_pushinteger(l[t], i);
-            err = lua_pcall(l[t], 1, LUA_MULTRET, 0);
+            err = luajr_pcall(l[t], 1, LUA_MULTRET, 0, tflags);
 
             // Check for errors
             if (err)
             {
                 std::lock_guard<std::mutex> lock { pm };
-                error_msg = lua_tostring(l[t], -1);
+                error_msg.assign(1024, ' ');
+                luajr_handle_lua_error(l[t], err, "lua_parallel 'func' execution", error_msg.data());
             }
             if (!error_msg.empty())
                 return;
@@ -148,14 +171,29 @@ extern "C" SEXP luajr_run_parallel(SEXP func, SEXP n, SEXP threads, SEXP pre)
         }
     };
 
-    // Create and assign work to the threads
-    std::vector<std::thread> thr;
-    for (unsigned int t = 0; t < l.size(); ++t)
-        thr.emplace_back(work, t);
+    // During parallel execution, it seems that input from the console is
+    // not available, even if there is only one thread going. So, don't drop
+    // into parallel execution if the number of threads is just one.
+    // (This allows the debugger to carry on working.)
+    if (l.size() > 1)
+    {
+        // Create and assign work to the threads
+        std::vector<std::thread> thr;
+        for (unsigned int t = 0; t < l.size(); ++t)
+            thr.emplace_back(work, t);
 
-    // Wait for threads to finish
-    for (unsigned int t = 0; t < thr.size(); ++t)
-        thr[t].join();
+        // Wait for threads to finish
+        for (unsigned int t = 0; t < thr.size(); ++t)
+            thr[t].join();
+    }
+    else
+    {
+        work(0);
+    }
+
+    // Collect any profiler data
+    for (unsigned int t = 0; t < l.size(); ++t)
+        luajr_profile_collect(l[t]);
 
     // Handle errors
     if (!error_msg.empty())
@@ -169,7 +207,7 @@ extern "C" SEXP luajr_run_parallel(SEXP func, SEXP n, SEXP threads, SEXP pre)
             for (unsigned int t = 0; t < l.size(); ++t)
                 lua_settop(l[t], 0);
         // Stop with error
-        Rf_error("Error running parallel task: %s", error_msg.c_str());
+        Rf_error("%s", error_msg.c_str());
     }
 
     // Assign computed values to list
